SystemOrganization addCategory: #'Seaside-REST-Core-Pragma'!
SystemOrganization addCategory: #'Seaside-REST-Core-RequestHandling'!
SystemOrganization addCategory: #'Seaside-REST-Core-Route'!

!GRPackage class methodsFor: '*seaside-rest-core' stamp: 'pmm 8/8/2010 11:15'!
seasideRestCore
	^ self new
		name: 'Seaside-REST-Core';
		description: 'Write RESTful web services using Seaside';
		addDependency: 'Seaside-Core';
		url: #seasideAddonsUrl;
		yourself! !

!GRPlatform methodsFor: '*seaside-rest-core' stamp: 'pmm 8/8/2010 10:27'!
argumentNamesOf: aMethod
	"Answer the argument names of a method as a sequenceable collection of strings. The method will be whatever you passed in #methodsWithHttpPragmaIn:do: as a first argument."
	self subclassResponsibility! !

!GRPlatform methodsFor: '*seaside-rest-core' stamp: 'pmm 8/8/2010 10:18'!
methodsWithHttpPragmaIn: aClass do: aFourArgumentBlock
	"Iterate over all the methods in the given class with an HTTP method pragma (see WARoute httpMethodNames) and evalute the given block for each of them.
	- the first argument is the method, no messages will be sent to this object and it won't be passed anywhere but to #argumentNamesOf:
	- the second argument is the method selector as a Symbol
	- third argument is the HTTP method name as a String, eg. 'GET'
	- the fourth argument is the value of the Path: paragma as String if present or nil otherwise, eg. '/foo/*.html'"
	self subclassResponsibility! !

WARequestFilter subclass: #WARestfulFilter
	instanceVariableNames: 'routes'
	classVariableNames: ''
	poolDictionaries: ''
	category: 'Seaside-REST-Core-RequestHandling'!

WARestfulFilter subclass: #WAPragmaBasedRestfulFilter
	instanceVariableNames: ''
	classVariableNames: ''
	poolDictionaries: ''
	category: 'Seaside-REST-Core-Pragma'!

!WAPragmaBasedRestfulFilter methodsFor: 'private' stamp: 'pmm 6/27/2010 12:11'!
createRoutes
	^ WAPragmaBasedRouteBuilder createRoutesFor: self class! !

!WARestfulFilter methodsFor: 'private' stamp: 'pmm 6/27/2010 12:06'!
createRoutes
	self subclassResponsibility! !

!WARestfulFilter methodsFor: 'request-handling' stamp: 'pmm 10/10/2010 21:04'!
executeResult: aRouteResult
	aRouteResult sendMessageTo: self! !

!WARestfulFilter methodsFor: 'request-handling' stamp: 'pmm 10/10/2010 21:04'!
handleFiltered: aRequestContext
	| result |
	result := self
		routeForMethod: aRequestContext request method
		consumer: aRequestContext consumer
		parameters: aRequestContext request url queryFields
		ifAbsent: [ self noRouteFound: aRequestContext ].
	self executeResult: result
	! !

!WARestfulFilter methodsFor: 'initialize-release' stamp: 'pmm 6/27/2010 22:56'!
initialize
	super initialize.
	self initializeRoutes! !

!WARestfulFilter methodsFor: 'initialize-release' stamp: 'pmm 6/27/2010 22:56'!
initializeRoutes
	routes := WAByMethodRoutes routes: self createRoutes! !

!WARestfulFilter methodsFor: 'private' stamp: 'pmm 6/27/2010 12:09'!
isImplemented: aString
	^ WARoute httpMethodNames includes: aString! !

!WARestfulFilter methodsFor: 'request-handling' stamp: 'pmm 6/27/2010 16:50'!
noRouteFound: aRequestContext
	aRequestContext responseGenerator
		notFound;
		respond! !

!WARestfulFilter methodsFor: 'private' stamp: 'pmm 6/27/2010 12:06'!
routeForMethod: aString consumer: aPathConsumer parameters: aDictionary ifAbsent: aNiliadicBlock
	^ routes
		routeForMethod: aString
		consumer: aPathConsumer
		parameters: aDictionary
		ifAbsent: aNiliadicBlock! !

WARequestHandler subclass: #WARestfulHandler
	instanceVariableNames: 'routes'
	classVariableNames: ''
	poolDictionaries: ''
	category: 'Seaside-REST-Core-RequestHandling'!

WARestfulHandler subclass: #WAPragmaBasedRestfulHandler
	instanceVariableNames: ''
	classVariableNames: ''
	poolDictionaries: ''
	category: 'Seaside-REST-Core-Pragma'!

!WAPragmaBasedRestfulHandler commentStamp: '<historical>' prior: 0!
A WAPragmaBasedRestfulHandler is a WARestfulHandler where routes are automatically created by looking at specific pragmas in the instance side methods.

One and only one of the following pragmas has to be present for a method to get picked up:
<COPY>
<DELETE>
<GET>
<MOVE>
<POST>
<PUT>
it will then by invoked by a matching HTTP request.

Optionally a <Path:> pragma may be present to customize the path to be used. The argument is an URL that can contain parameters and placeholders. Placeholders can either be argument indices or argument names.

Examples
<Path: '/{databaseName}/_all_docs'>
<Path: '/{1}/_all_docs'>!

!WAPragmaBasedRestfulHandler methodsFor: 'private' stamp: 'pmm 6/27/2010 12:11'!
createRoutes
	^ WAPragmaBasedRouteBuilder createRoutesFor: self class! !

!WARestfulHandler methodsFor: 'private' stamp: 'pmm 9/1/2009 17:45'!
createRoutes
	self subclassResponsibility! !

!WARestfulHandler methodsFor: 'request-handling' stamp: 'pmm 10/10/2010 21:04'!
executeResult: aRouteResult
	aRouteResult sendMessageTo: self! !

!WARestfulHandler methodsFor: 'request-handling' stamp: 'pmm 10/10/2010 21:06'!
handleFiltered: aRequestContext
	| result |
	result := self
		routeForMethod: aRequestContext request method
		consumer: aRequestContext consumer
		parameters: aRequestContext request url queryFields
		ifAbsent: [ self noRouteFound: aRequestContext ].
	self executeResult: result! !

!WARestfulHandler methodsFor: 'initialize-release' stamp: 'pmm 6/27/2010 22:56'!
initialize
	super initialize.
	self initializeRoutes! !

!WARestfulHandler methodsFor: 'initialize-release' stamp: 'pmm 6/27/2010 22:56'!
initializeRoutes
	routes := WAByMethodRoutes routes: self createRoutes! !

!WARestfulHandler methodsFor: 'private' stamp: 'pmm 6/27/2010 12:09'!
isImplemented: aString
	^ WARoute httpMethodNames includes: aString! !

!WARestfulHandler methodsFor: 'request-handling' stamp: 'pmm 6/27/2010 12:08'!
noRouteFound: aRequestContext
	super handleFiltered: aRequestContext! !

!WARestfulHandler methodsFor: 'private' stamp: 'pmm 9/8/2009 22:24'!
routeForMethod: aString consumer: aPathConsumer parameters: aDictionary ifAbsent: aNiliadicBlock
	^ routes
		routeForMethod: aString
		consumer: aPathConsumer
		parameters: aDictionary
		ifAbsent: aNiliadicBlock! !

WAError subclass: #WAArgumentNotFoundError
	instanceVariableNames: ''
	classVariableNames: ''
	poolDictionaries: ''
	category: 'Seaside-REST-Core-Route'!

WAObject subclass: #WAByMethodRoutes
	instanceVariableNames: 'routes'
	classVariableNames: ''
	poolDictionaries: ''
	category: 'Seaside-REST-Core-Route'!

!WAByMethodRoutes commentStamp: '<historical>' prior: 0!
A WAByMethodRoutes is a collection of routes first organized by HTTP method and then by number of URL path elements. It allows users to look up possible routes for a given HTTP request.

See also WARouteResult and WARouteContainer.

Instance Variables
	routes:		<Dictionary<String, SortedCollection<WARouteContainer>>

routes
	- the routes keyed by HTTP method, sourted by number of path elements
!

!WAByMethodRoutes class methodsFor: 'instance creation' stamp: 'pmm 9/8/2009 22:12'!
routes: aCollectionOfRoutes
	^ self basicNew initializeWithRoutes: aCollectionOfRoutes! !

!WAByMethodRoutes methodsFor: 'initialize-release' stamp: 'pmm 9/8/2009 22:33'!
initializeWithRoutes: aCollectionOfRoutes
	| routeBuilder |
	super initialize.
	routeBuilder := Dictionary new.
	aCollectionOfRoutes do: [ :each |
		| methodRoutes subRoutes |
		methodRoutes := routeBuilder
			at: each method
			ifAbsentPut: [ Dictionary new ].
		subRoutes := methodRoutes at: each pathElementCount ifAbsentPut: [ OrderedCollection new ].
		subRoutes add: each ].
	routes := Dictionary new.
	routeBuilder keysAndValuesDo: [ :method :subroutes |
		| methodRoutes |
		methodRoutes := routes
			at: method
			ifAbsentPut: [ SortedCollection sortBlock: [ :a :b |
					a pathElementCount <= b pathElementCount ] ].
		subroutes valuesDo: [ :each |
			methodRoutes add: (WARouteCountainer routes: each) ] ]! !

!WAByMethodRoutes methodsFor: 'public' stamp: 'pmm 9/9/2009 19:54'!
routeForMethod: aString consumer: aPathConsumer parameters: aDictionary ifAbsent: aNiliadicBlock
	| routesForMethod routesStream route count elements |
	routesForMethod := routes at: aString ifAbsent: [ ^ aNiliadicBlock value ].
	elements := OrderedCollection new.
	routesStream := routesForMethod readStream.
	routesStream atEnd ifTrue: [ ^ aNiliadicBlock value ].
	route := routesStream next.
	count := 0.
	[ aPathConsumer atEnd ]  whileFalse: [
		elements add: aPathConsumer next.
		count := count + 1.
		[ route pathElementCount < count ] whileTrue: [
			routesStream atEnd ifTrue: [ ^ aNiliadicBlock value ].
			route := routesStream next ] ].
	count = route pathElementCount
		ifFalse: [ ^ aNiliadicBlock value ].
	^ WARouteResult
		route: (route
			routeForElements: elements
			parameters: aDictionary
			ifAbsent: [ ^ aNiliadicBlock value ])
		elements: elements asArray
		parameters: aDictionary! !

WAObject subclass: #WAMatch
	instanceVariableNames: ''
	classVariableNames: ''
	poolDictionaries: ''
	category: 'Seaside-REST-Core-Route'!

!WAMatch class methodsFor: 'private' stamp: 'pmm 6/13/2010 10:01'!
argument: aString notFoundInd: aCollectionOfStrings
	WAArgumentNotFoundError signal: 'argument "', aString, '" was not found in ', aCollectionOfStrings greaseString! !

!WAMatch class methodsFor: 'private' stamp: 'pmm 6/13/2010 09:49'!
targetIndexOf: aString arguments: aCollectionOfStrings
	aString do: [ :each |
		each isDigit ifFalse: [
			| index |
			index := aCollectionOfStrings indexOf: aString.
			index isZero ifTrue: [
				self argument: aString notFoundInd: aCollectionOfStrings ].
			^index ] ].
	^ aString asNumber! !

WAMatch subclass: #WAMimeTypeMatch
	instanceVariableNames: ''
	classVariableNames: ''
	poolDictionaries: ''
	category: 'Seaside-REST-Core-Route'!

WAMimeTypeMatch subclass: #WAFullMimeTypeMatch
	instanceVariableNames: 'main sub'
	classVariableNames: ''
	poolDictionaries: ''
	category: 'Seaside-REST-Core-Route'!

!WAFullMimeTypeMatch class methodsFor: 'instance creation' stamp: 'pmm 10/6/2010 11:42'!
main: aMainString sub: aSubString
	^self basicNew initializeWithMain: aMainString sub: aSubString! !

!WAFullMimeTypeMatch methodsFor: 'initialization' stamp: 'pmm 10/6/2010 11:42'!
initializeWithMain: aMainString sub: aSubString
	self initialize.
	main := aMainString.
	sub := aSubString! !

!WAFullMimeTypeMatch methodsFor: 'testing' stamp: 'pmm 10/6/2010 12:10'!
matches: aMimeType
	^(aMimeType main = '*'
		or: [ aMimeType main = main ])
		and: [
			aMimeType sub = '*'
				or: [ aMimeType sub = sub ]]! !

!WAFullMimeTypeMatch methodsFor: 'accessing' stamp: 'pmm 10/6/2010 12:37'!
priority
	^3! !

WAMimeTypeMatch subclass: #WAMainMimeTypeMatch
	instanceVariableNames: 'main'
	classVariableNames: ''
	poolDictionaries: ''
	category: 'Seaside-REST-Core-Route'!

!WAMainMimeTypeMatch class methodsFor: 'instance creation' stamp: 'pmm 10/6/2010 11:44'!
main: aMainString
	^self basicNew initializeWithMain: aMainString! !

!WAMainMimeTypeMatch methodsFor: 'initialization' stamp: 'pmm 10/6/2010 11:44'!
initializeWithMain: aMainString
	self initialize.
	main := aMainString! !

!WAMainMimeTypeMatch methodsFor: 'testing' stamp: 'pmm 10/6/2010 12:09'!
matches: aMimeType
	^aMimeType main = '*'
		or: [ aMimeType main = main ]! !

!WAMainMimeTypeMatch methodsFor: 'accessing' stamp: 'pmm 10/6/2010 12:37'!
priority
	^2! !

!WAMimeTypeMatch class methodsFor: 'instance creation' stamp: 'pmm 10/6/2010 11:49'!
fromMimeType: aMimeType
	^ aMimeType main = '*'
		ifTrue: [
			aMimeType sub = '*'
				ifTrue: [ WAWildcardMimeTypeMatch new ]
				ifFalse: [ WASubMimeTypeMatch
					sub: aMimeType sub ] ]
		ifFalse: [
			aMimeType sub = '*'
				ifTrue: [ WAMainMimeTypeMatch
					main: aMimeType main ]
				ifFalse: [ WAFullMimeTypeMatch
					main: aMimeType main
					sub: aMimeType sub ]]! !

!WAMimeTypeMatch methodsFor: 'testing' stamp: 'pmm 10/6/2010 12:08'!
matches: aMimeType
	self subclassResponsibility! !

!WAMimeTypeMatch methodsFor: 'accessing' stamp: 'pmm 10/6/2010 11:31'!
priority
	self subclassResponsibility! !

WAMimeTypeMatch subclass: #WASubMimeTypeMatch
	instanceVariableNames: 'sub'
	classVariableNames: ''
	poolDictionaries: ''
	category: 'Seaside-REST-Core-Route'!

!WASubMimeTypeMatch class methodsFor: 'instance creation' stamp: 'pmm 10/6/2010 12:06'!
sub: aSubString
	^self basicNew initializeWithSub: aSubString! !

!WASubMimeTypeMatch methodsFor: 'initialization' stamp: 'pmm 10/6/2010 11:43'!
initializeWithSub: aSubString
	self initialize.
	sub := aSubString! !

!WASubMimeTypeMatch methodsFor: 'testing' stamp: 'pmm 10/6/2010 12:09'!
matches: aMimeType
	^aMimeType sub = '*'
		or: [ aMimeType sub = sub ]! !

!WASubMimeTypeMatch methodsFor: 'accessing' stamp: 'pmm 10/6/2010 12:05'!
priority
	^2! !

WAMimeTypeMatch subclass: #WAWildcardMimeTypeMatch
	instanceVariableNames: ''
	classVariableNames: ''
	poolDictionaries: ''
	category: 'Seaside-REST-Core-Route'!

!WAWildcardMimeTypeMatch methodsFor: 'testing' stamp: 'pmm 10/6/2010 12:08'!
matches: aMimeType
	^true! !

!WAWildcardMimeTypeMatch methodsFor: 'accessing' stamp: 'pmm 10/6/2010 12:04'!
priority
	^1! !

WAMatch subclass: #WAPPathMatch
	instanceVariableNames: ''
	classVariableNames: ''
	poolDictionaries: ''
	category: 'Seaside-REST-Core-Route'!

WAPPathMatch subclass: #WAExactPathMatch
	instanceVariableNames: 'element'
	classVariableNames: ''
	poolDictionaries: ''
	category: 'Seaside-REST-Core-Route'!

!WAExactPathMatch class methodsFor: 'instance creation' stamp: 'pmm 9/7/2009 00:07'!
fromString: aString
	^ self basicNew inititializeWith: aString! !

!WAExactPathMatch class methodsFor: 'instance creation' stamp: 'pmm 6/13/2010 09:30'!
fromString: aString index: anInteger arguments: aCollectionOfStrings
	^ self fromString: aString! !

!WAExactPathMatch methodsFor: 'accessing' stamp: 'pmm 9/8/2009 18:30'!
argumentCount
	^ 0! !

!WAExactPathMatch methodsFor: 'initialize-release' stamp: 'pmm 9/7/2009 00:07'!
inititializeWith: aString
	super initialize.
	element := aString! !

!WAExactPathMatch methodsFor: 'public' stamp: 'pmm 9/10/2009 09:10'!
mapArgumentsWithElements: aCollection into: anArray
	"ignore"! !

!WAExactPathMatch methodsFor: 'testing' stamp: 'pmm 9/7/2009 00:07'!
matches: aString
	^ element = aString! !

!WAExactPathMatch methodsFor: 'accessing' stamp: 'pmm 6/26/2010 17:45'!
priority
	^ 3! !

!WAPPathMatch class methodsFor: 'instance creation' stamp: 'pmm 6/27/2010 00:28'!
fromString: aString index: anInteger arguments: aCollectionOfStrings
	^ (self matchClassFor: aString)
		fromString: aString index: anInteger arguments: aCollectionOfStrings! !

!WAPPathMatch class methodsFor: 'private' stamp: 'pmm 6/27/2010 10:18'!
matchClassFor: aString
	aString isEmpty ifTrue: [ ^ WAExactPathMatch  ].
	aString first = ${
		ifTrue: [
			| index |
			index := aString indexOf: $} startingAt: 2.
			index = aString size ifTrue: [ ^ WAFullPlaceholderPathMatch ].
			index isZero ifFalse: [ ^ WASuffixPlaceholderPathMatch ] ].
	(aString last = $}
		and: [ (aString indexOf: ${) isZero not ])
			ifTrue: [ ^ WAPrefixPlaceholderPathMatch ].
	^ WAExactPathMatch! !

!WAPPathMatch methodsFor: 'accessing' stamp: 'pmm 9/8/2009 18:30'!
argumentCount
	self subclassResponsibility! !

!WAPPathMatch methodsFor: 'public' stamp: 'pmm 9/10/2009 09:10'!
mapArgumentsWithElements: aCollection into: anArray
	self subclassResponsibility! !

!WAPPathMatch methodsFor: 'testing' stamp: 'pmm 9/6/2009 23:57'!
matches: aString
	self subclassResponsibility! !

!WAPPathMatch methodsFor: 'accessing' stamp: 'pmm 9/6/2009 21:49'!
priority
	self subclassResponsibility! !

WAPPathMatch subclass: #WAPlaceholderPathMatch
	instanceVariableNames: 'sourceIndex targetIndex'
	classVariableNames: ''
	poolDictionaries: ''
	category: 'Seaside-REST-Core-Route'!

WAPlaceholderPathMatch subclass: #WAFullPlaceholderPathMatch
	instanceVariableNames: ''
	classVariableNames: ''
	poolDictionaries: ''
	category: 'Seaside-REST-Core-Route'!

!WAFullPlaceholderPathMatch class methodsFor: 'instance creation' stamp: 'pmm 6/27/2010 10:09'!
fromString: aString index: anInteger arguments: aCollectionOfStrings
	^ self
		source: anInteger
		target: (self
			targetIndexOf: (aString copyFrom: 2 to: aString size - 1)
			arguments: aCollectionOfStrings)! !

!WAFullPlaceholderPathMatch class methodsFor: 'instance creation' stamp: 'pmm 6/27/2010 10:12'!
source: anFirstInteger target: aSecondInteger
	^ self basicNew initializeWithSourceIndex: anFirstInteger targetIndex: aSecondInteger! !

!WAFullPlaceholderPathMatch methodsFor: 'public' stamp: 'pmm 6/30/2010 07:27'!
mapArgumentsWithElements: aCollection into: anArray
	anArray
		at: targetIndex
		put: (aCollection at: sourceIndex)! !

!WAFullPlaceholderPathMatch methodsFor: 'testing' stamp: 'pmm 6/26/2010 17:45'!
matches: aString
	^ true! !

!WAFullPlaceholderPathMatch methodsFor: 'accessing' stamp: 'pmm 6/26/2010 17:44'!
priority
	^ 1! !

!WAPlaceholderPathMatch methodsFor: 'accessing' stamp: 'pmm 9/8/2009 18:30'!
argumentCount
	^ 1! !

!WAPlaceholderPathMatch methodsFor: 'initialize-release' stamp: 'pmm 9/10/2009 09:18'!
initializeWithSourceIndex: anFirstInteger targetIndex: aSecondInteger
	super initialize.
	sourceIndex := anFirstInteger.
	targetIndex := aSecondInteger! !

!WAPlaceholderPathMatch methodsFor: 'accessing' stamp: 'pmm 9/10/2009 09:05'!
sourceIndex
	^ sourceIndex! !

!WAPlaceholderPathMatch methodsFor: 'accessing' stamp: 'pmm 9/10/2009 09:11'!
targetIndex
	^ targetIndex! !

WAPlaceholderPathMatch subclass: #WAPrefixPlaceholderPathMatch
	instanceVariableNames: 'prefix'
	classVariableNames: ''
	poolDictionaries: ''
	category: 'Seaside-REST-Core-Route'!

!WAPrefixPlaceholderPathMatch class methodsFor: 'instance creation' stamp: 'pmm 6/27/2010 10:20'!
fromString: aString index: anInteger arguments: aCollectionOfStrings
	| prefix |
	prefix := aString copyUpTo: ${.
	^ self
		source: anInteger
		target: (self
			targetIndexOf: (aString copyFrom: prefix size + 2 to: aString size - 1)
			arguments: aCollectionOfStrings)
		prefix: prefix! !

!WAPrefixPlaceholderPathMatch class methodsFor: 'instance creation' stamp: 'pmm 6/27/2010 10:14'!
source: anFirstInteger target: aSecondInteger prefix: aString
	^ self basicNew
		initializeWithSourceIndex: anFirstInteger
		targetIndex: aSecondInteger
		prefix: aString! !

!WAPrefixPlaceholderPathMatch methodsFor: 'initialize-release' stamp: 'pmm 6/27/2010 10:14'!
initializeWithSourceIndex: anFirstInteger targetIndex: aSecondInteger prefix: aString
	super initializeWithSourceIndex: anFirstInteger targetIndex: aSecondInteger.
	prefix := aString! !

!WAPrefixPlaceholderPathMatch methodsFor: 'public' stamp: 'pmm 6/30/2010 08:43'!
mapArgumentsWithElements: aCollection into: anArray
	| value |
	value := aCollection at: sourceIndex.
	anArray
		at: targetIndex
		put: (value copyFrom: prefix size + 1 to: value size)! !

!WAPrefixPlaceholderPathMatch methodsFor: 'testing' stamp: 'pmm 6/26/2010 17:46'!
matches: aString
	^ (aString size > prefix size)
		and: [ aString beginsWith: prefix ]! !

!WAPrefixPlaceholderPathMatch methodsFor: 'accessing' stamp: 'pmm 6/26/2010 17:44'!
priority
	^ 2! !

WAPlaceholderPathMatch subclass: #WASuffixPlaceholderPathMatch
	instanceVariableNames: 'suffix'
	classVariableNames: ''
	poolDictionaries: ''
	category: 'Seaside-REST-Core-Route'!

!WASuffixPlaceholderPathMatch class methodsFor: 'instance creation' stamp: 'pmm 6/29/2010 08:16'!
fromString: aString index: anInteger arguments: aCollectionOfStrings
	| suffix |
	suffix := aString copyAfter: $}.
	^ self
		source: anInteger
		target: (self
			targetIndexOf: (aString copyFrom: 2 to: aString size - suffix size - 1)
			arguments: aCollectionOfStrings)
		suffix: suffix! !

!WASuffixPlaceholderPathMatch class methodsFor: 'instance creation' stamp: 'pmm 6/27/2010 10:14'!
source: anFirstInteger target: aSecondInteger suffix: aString
	^ self basicNew
		initializeWithSourceIndex: anFirstInteger
		targetIndex: aSecondInteger
		suffix: aString! !

!WASuffixPlaceholderPathMatch methodsFor: 'initialize-release' stamp: 'pmm 6/27/2010 10:14'!
initializeWithSourceIndex: anFirstInteger targetIndex: aSecondInteger suffix: aString
	super initializeWithSourceIndex: anFirstInteger targetIndex: aSecondInteger.
	suffix := aString! !

!WASuffixPlaceholderPathMatch methodsFor: 'public' stamp: 'pmm 6/30/2010 08:57'!
mapArgumentsWithElements: aCollection into: anArray
	| value |
	value := aCollection at: sourceIndex.
	anArray
		at: targetIndex
		put: (value copyFrom: 1 to: value size - suffix size)! !

!WASuffixPlaceholderPathMatch methodsFor: 'testing' stamp: 'pmm 6/26/2010 17:47'!
matches: aString
	^ (aString size > suffix size)
		and: [ aString endsWith: suffix ]! !

!WASuffixPlaceholderPathMatch methodsFor: 'accessing' stamp: 'pmm 6/26/2010 17:44'!
priority
	^ 2! !

WAMatch subclass: #WAParameterMatch
	instanceVariableNames: 'name'
	classVariableNames: ''
	poolDictionaries: ''
	category: 'Seaside-REST-Core-Route'!

WAParameterMatch subclass: #WAExactParameterMatch
	instanceVariableNames: 'value'
	classVariableNames: ''
	poolDictionaries: ''
	category: 'Seaside-REST-Core-Route'!

!WAExactParameterMatch class methodsFor: 'instance creation' stamp: 'pmm 6/13/2010 09:57'!
name: aNameString value: aValueString arguments: aCollectionOfStrings 
	^ self basicNew initalizeWithName: aNameString value: aValueString! !

!WAExactParameterMatch methodsFor: 'accessing' stamp: 'pmm 9/8/2009 18:29'!
argumentCount
	^ 0! !

!WAExactParameterMatch methodsFor: 'initialize-release' stamp: 'pmm 9/8/2009 13:15'!
initalizeWithName: aNameString value: aValueString
	super initalizeWithName: aNameString.
	value := aValueString! !

!WAExactParameterMatch methodsFor: 'public' stamp: 'pmm 9/9/2009 07:06'!
mapArgumentsWithParameters: aDictionary into: anArray
	"ignore"! !

!WAExactParameterMatch methodsFor: 'testing' stamp: 'pmm 9/8/2009 13:08'!
matchesName: aNameString value: aValueString
	^name = aNameString
		and: [ value = aValueString ]! !

!WAParameterMatch class methodsFor: 'private' stamp: 'pmm 9/8/2009 13:20'!
isPattern: aString
	^ aString isEmpty not
		and: [ aString first = ${
		and: [ aString last = $} ] ]! !

!WAParameterMatch class methodsFor: 'instance creation' stamp: 'pmm 6/13/2010 09:57'!
name: aKeyString value: aValueString arguments: aCollectionOfStrings 
	^ ((self isPattern: aValueString)
		ifTrue: [ WAPlaceholderParameterMatch ]
		ifFalse: [ WAExactParameterMatch ]) name: aKeyString value: aValueString arguments: aCollectionOfStrings ! !

!WAParameterMatch methodsFor: 'accessing' stamp: 'pmm 9/8/2009 18:29'!
argumentCount
	self subclassResponsibility! !

!WAParameterMatch methodsFor: 'initialize-release' stamp: 'pmm 9/8/2009 13:15'!
initalizeWithName: aString
	super initialize.
	name := aString! !

!WAParameterMatch methodsFor: 'public' stamp: 'pmm 9/9/2009 07:05'!
mapArgumentsWithParameters: aDictionary into: anArray
	self subclassResponsibility! !

!WAParameterMatch methodsFor: 'testing' stamp: 'pmm 9/8/2009 13:06'!
matchesName: aNameString value: aValueString
	self subclassResponsibility! !

WAParameterMatch subclass: #WAPlaceholderParameterMatch
	instanceVariableNames: 'index'
	classVariableNames: ''
	poolDictionaries: ''
	category: 'Seaside-REST-Core-Route'!

!WAPlaceholderParameterMatch class methodsFor: 'instance creation' stamp: 'pmm 9/8/2009 13:24'!
name: aString index: anInteger
	^ self basicNew initalizeWithName: aString index: anInteger! !

!WAPlaceholderParameterMatch class methodsFor: 'instance creation' stamp: 'pmm 6/13/2010 09:51'!
name: aNameString value: aValueString arguments: aCollectionOfStrings
	^ self
		name: aNameString
		index: (self
			targetIndexOf: (aValueString copyFrom: 2 to: aValueString size - 1)
			arguments: aCollectionOfStrings)! !

!WAPlaceholderParameterMatch methodsFor: 'accessing' stamp: 'pmm 9/8/2009 18:30'!
argumentCount
	^ 1! !

!WAPlaceholderParameterMatch methodsFor: 'accessing' stamp: 'pmm 6/13/2010 09:53'!
index
	^ index! !

!WAPlaceholderParameterMatch methodsFor: 'initialize-release' stamp: 'pmm 9/8/2009 13:18'!
initalizeWithName: aString index: anInteger
	super initalizeWithName: aString.
	index := anInteger! !

!WAPlaceholderParameterMatch methodsFor: 'public' stamp: 'pmm 9/9/2009 07:06'!
mapArgumentsWithParameters: aDictionary into: anArray
	anArray at: index put: (aDictionary at: name)! !

!WAPlaceholderParameterMatch methodsFor: 'testing' stamp: 'pmm 9/8/2009 13:08'!
matchesName: aNameString value: aValueString
	^name = aNameString! !

WAObject subclass: #WAPragmaBasedRouteBuilder
	instanceVariableNames: ''
	classVariableNames: ''
	poolDictionaries: ''
	category: 'Seaside-REST-Core-Pragma'!

!WAPragmaBasedRouteBuilder class methodsFor: 'public' stamp: 'pmm 8/8/2010 10:08'!
createRoutesFor: aClass
	| routes |
	routes := OrderedCollection new.
	GRPlatform current methodsWithHttpPragmaIn: aClass do: [ :method :selector :httpMethodName :path |
			routes add: (path isNil
				ifTrue: [
					WASimpleRoute
						method: httpMethodName
						selector: selector ]
				ifFalse: [
					| arguments |
					arguments := GRPlatform current argumentNamesOf: method.
					WAComplexRoute
						method: httpMethodName
						selector: selector
						pattern: path
						arguments: arguments ] ) ].
	^ routes! !

WAObject subclass: #WARoute
	instanceVariableNames: 'method selector produces consumes'
	classVariableNames: ''
	poolDictionaries: ''
	category: 'Seaside-REST-Core-Route'!

!WARoute commentStamp: 'pmm 10/6/2010 12:22' prior: 0!
I am a method that can be sent if I match a request. Method arguments can be taken from the request.

Instance Variables:
	method		<String>
	selector	<Symbol>
		
method
	- The HTTP method on which to follow this route, eg. 'GET'

selector
	- The selector to perform, eg. #index!

WARoute subclass: #WAComplexRoute
	instanceVariableNames: 'pathMatches parameterMatches'
	classVariableNames: ''
	poolDictionaries: ''
	category: 'Seaside-REST-Core-Route'!

!WAComplexRoute class methodsFor: 'examples' stamp: 'pmm 9/6/2009 21:36'!
example
	^self
		get: '/{3}/_all_docs?startkey={1}&limit={2}'
		selector: #getAllDocumentsStartingAt:limitedTo:inDatabase:! !

!WAComplexRoute class methodsFor: 'instance creation' stamp: 'pmm 6/13/2010 09:40'!
method: aMethodString selector: aSymbol pattern: aPatternString
	^self
		method: aMethodString
		selector: aSymbol
		url: (WAUrl absolute: aPatternString)
		arguments: #()! !

!WAComplexRoute class methodsFor: 'instance creation' stamp: 'pmm 6/13/2010 09:39'!
method: aMethodString selector: aSymbol pattern: aPatternString arguments: aCollectionOfStrings
	^self
		method: aMethodString
		selector: aSymbol
		url: (WAUrl absolute: aPatternString)
		arguments: aCollectionOfStrings! !

!WAComplexRoute class methodsFor: 'instance creation' stamp: 'pmm 6/13/2010 09:38'!
method: aString selector: aSymbol url: aUrl arguments: aCollectionOfStrings
	^self basicNew initializeWithMethod: aString selector: aSymbol url: aUrl arguments: aCollectionOfStrings! !

!WAComplexRoute methodsFor: 'accessing' stamp: 'pmm 9/8/2009 18:32'!
argumentCount
	| count |
	count := 0.
	pathMatches do: [ :each |
		count := count + each argumentCount ].
	parameterMatches valuesDo: [ :each |
		count := count + each argumentCount ].
	^ count ! !

!WAComplexRoute methodsFor: 'initialize-release' stamp: 'pmm 6/13/2010 09:51'!
initializeWithMethod: aString selector: aSymbol url: aUrl arguments: aCollectionOfStrings
	super initializeWithMethod: aString selector: aSymbol.
	pathMatches := aUrl path withIndexCollect: [ :each :index |
		WAPPathMatch fromString: each index: index arguments: aCollectionOfStrings ].
	parameterMatches := Dictionary new.
	aUrl parameters keysAndValuesDo: [ :key :value |
			parameterMatches
				at: key
				put: (WAParameterMatch name: key value: value arguments: aCollectionOfStrings) ]! !

!WAComplexRoute methodsFor: 'public' stamp: 'pmm 9/10/2009 09:10'!
mapArgumentsWithElements: anStringArray parameters: aDictionary into: anArray
	pathMatches do: [ :each |
		each mapArgumentsWithElements: anStringArray into: anArray ].
	parameterMatches valuesDo: [ :each |
		each mapArgumentsWithParameters: aDictionary into: anArray ]! !

!WAComplexRoute methodsFor: 'testing' stamp: 'pmm 9/8/2009 17:13'!
matchesParameters: aDictionary
	parameterMatches size = aDictionary size
		ifFalse: [ ^ false ].
	parameterMatches keysAndValuesDo: [ :name :match |
		| value |
		value := aDictionary at: name ifAbsent: [ ^ false ].
		(match matchesName: name value: value)
			ifFalse: [ ^ false ] ].
	^ true! !

!WAComplexRoute methodsFor: 'testing' stamp: 'pmm 9/7/2009 00:12'!
matchesPath: aCollectionOfStrings
	pathMatches size = aCollectionOfStrings size
		ifFalse: [ ^ false ].
	pathMatches with: aCollectionOfStrings do: [ :match :each |
		(match matches: each)
			ifFalse: [ ^ false ] ].
	^ true! !

!WAComplexRoute methodsFor: 'accessing' stamp: 'pmm 9/6/2009 22:09'!
pathElementCount
	^ pathMatches size! !

!WAComplexRoute methodsFor: 'accessing' stamp: 'pmm 9/7/2009 00:27'!
priority
	^ pathMatches inject: 0 into: [ :sum :each |
		sum + each priority ]! !

!WARoute class methodsFor: 'instance creation' stamp: 'pmm 9/6/2009 23:52'!
copy: aSymbol
	^ WASimpleRoute method: 'COPY' selector: aSymbol! !

!WARoute class methodsFor: 'instance creation' stamp: 'pmm 9/6/2009 23:52'!
delete: aSymbol
	^ WASimpleRoute method: 'DELETE' selector: aSymbol! !

!WARoute class methodsFor: 'instance creation' stamp: 'pmm 9/9/2009 07:24'!
delete: aString selector: aSymbol
	^ WAComplexRoute
		method: 'DELETE'
		selector: aSymbol
		pattern: aString! !

!WARoute class methodsFor: 'instance creation' stamp: 'pmm 9/6/2009 23:52'!
get: aSymbol
	^ WASimpleRoute method: 'GET' selector: aSymbol! !

!WARoute class methodsFor: 'instance creation' stamp: 'pmm 9/7/2009 00:14'!
get: aString selector: aSymbol
	^ WAComplexRoute
		method: 'GET'
		selector: aSymbol
		pattern: aString! !

!WARoute class methodsFor: 'instance creation' stamp: 'pmm 5/30/2010 20:25'!
httpMethodNames
	^ #(
		COPY
		DELETE
		GET
		MOVE
		POST
		PUT
	)! !

!WARoute class methodsFor: 'instance creation' stamp: 'pmm 9/6/2009 23:52'!
move: aSymbol
	^ WASimpleRoute method: 'MOVE' selector: aSymbol! !

!WARoute class methodsFor: 'instance creation' stamp: 'pmm 9/6/2009 23:52'!
post: aSymbol
	^ WASimpleRoute method: 'POST' selector: aSymbol! !

!WARoute class methodsFor: 'instance creation' stamp: 'pmm 9/9/2009 07:23'!
post: aString selector: aSymbol
	^ WAComplexRoute
		method: 'POST'
		selector: aSymbol
		pattern: aString! !

!WARoute class methodsFor: 'instance creation' stamp: 'pmm 9/6/2009 23:52'!
put: aSymbol
	^ WASimpleRoute method: 'PUT' selector: aSymbol! !

!WARoute class methodsFor: 'instance creation' stamp: 'pmm 9/9/2009 07:23'!
put: aString selector: aSymbol
	^ WAComplexRoute
		method: 'PUT'
		selector: aSymbol
		pattern: aString! !

!WARoute methodsFor: 'accessing' stamp: 'pmm 9/8/2009 18:29'!
argumentCount
	self subclassResponsibility! !

!WARoute methodsFor: 'testing' stamp: 'pmm 10/6/2010 12:53'!
consumes: aMimeType
	^consumes anySatisfy: [ :each |
		each matches: aMimeType ]! !

!WARoute methodsFor: 'initialize-release' stamp: 'pmm 9/6/2009 22:06'!
initializeWithMethod: aString selector: aSymbol
	super initialize.
	method := aString.
	selector := aSymbol! !

!WARoute methodsFor: 'public' stamp: 'pmm 9/9/2009 07:00'!
mapArgumentsWithElements: anStringArray parameters: aDictionary into: anArray
	self subclassResponsibility! !

!WARoute methodsFor: 'testing' stamp: 'pmm 9/8/2009 17:13'!
matchesParameters: aDictionary
	self subclassResponsibility! !

!WARoute methodsFor: 'testing' stamp: 'pmm 9/7/2009 00:28'!
matchesPath: aCollectionOfStrings
	self subclassResponsibility! !

!WARoute methodsFor: 'accessing' stamp: 'pmm 9/6/2009 22:05'!
method
	^ method! !

!WARoute methodsFor: 'accessing' stamp: 'pmm 9/6/2009 22:07'!
pathElementCount
	self subclassResponsibility! !

!WARoute methodsFor: 'accessing' stamp: 'pmm 9/7/2009 00:25'!
priority
	self subclassResponsibility! !

!WARoute methodsFor: 'testing' stamp: 'pmm 10/6/2010 12:53'!
produces: aMimeType
	^produces anySatisfy: [ :each |
		each matches: aMimeType ]! !

!WARoute methodsFor: 'accessing' stamp: 'pmm 9/6/2009 22:06'!
selector
	^ selector! !

!WARoute methodsFor: 'public' stamp: 'pmm 9/9/2009 19:27'!
sendSelectorWith: anArrayOfArguments to: anObject
	^anObject perform: selector withArguments: anArrayOfArguments! !

WARoute subclass: #WASimpleRoute
	instanceVariableNames: 'count'
	classVariableNames: ''
	poolDictionaries: ''
	category: 'Seaside-REST-Core-Route'!

!WASimpleRoute commentStamp: 'pmm 10/6/2010 12:26' prior: 0!
I match only on an exact number of path elements and no URL parameters. The path elements are turned into method arguments.

Instance Variables:
	count	<Integer>
		
count
	- The exact number of path elements that have to be present.!

!WASimpleRoute class methodsFor: 'instance creation' stamp: 'pmm 9/1/2009 17:43'!
method: aString count: anInteger selector: aSymbol
	^ self basicNew initializeWithMethod: aString count: anInteger selector: aSymbol! !

!WASimpleRoute class methodsFor: 'instance creation' stamp: 'pmm 9/1/2009 17:54'!
method: aString selector: aSymbol
	^ self basicNew
		initializeWithMethod: aString
		count: (aSymbol count: [ :each | each = $: ])
		selector: aSymbol! !

!WASimpleRoute methodsFor: 'accessing' stamp: 'pmm 9/8/2009 18:29'!
argumentCount
	^ self pathElementCount! !

!WASimpleRoute methodsFor: 'initialize-release' stamp: 'pmm 9/6/2009 22:07'!
initializeWithMethod: aString count: anInteger selector: aSymbol
	super initializeWithMethod: aString selector: aSymbol.
	count := anInteger! !

!WASimpleRoute methodsFor: 'public' stamp: 'pmm 9/9/2009 07:01'!
mapArgumentsWithElements: anStringArray parameters: aDictionary into: anArray
	1 to: self argumentCount do: [ :index |
		anArray at: index put: (anStringArray at: index) ]! !

!WASimpleRoute methodsFor: 'testing' stamp: 'pmm 9/8/2009 17:13'!
matchesParameters: aDictionary
	^ aDictionary isEmpty! !

!WASimpleRoute methodsFor: 'testing' stamp: 'pmm 9/8/2009 13:02'!
matchesPath: aCollectionOfStrings
	^ self pathElementCount = aCollectionOfStrings size! !

!WASimpleRoute methodsFor: 'accessing' stamp: 'pmm 9/6/2009 21:55'!
pathElementCount
	^ count! !

!WASimpleRoute methodsFor: 'accessing' stamp: 'pmm 9/7/2009 00:25'!
priority
	^ self pathElementCount! !

WAObject subclass: #WARouteCountainer
	instanceVariableNames: 'pathElementCount routes'
	classVariableNames: ''
	poolDictionaries: ''
	category: 'Seaside-REST-Core-Route'!

!WARouteCountainer commentStamp: '<historical>' prior: 0!
A WARouteCountainer is a collection of routes that all accept the same number of path elements ordered by priority.

Instance Variables
	pathElementCount:		<Integer>
	routes:		<SortedCollection<WARoute>>

pathElementCount
	- the number of path elements that any of the routes in this container accepts

routes
	- the routes ordered by priority
!

!WARouteCountainer class methodsFor: 'instance creation' stamp: 'pmm 9/8/2009 21:49'!
pathElementCount: anInteger
	^ self basicNew initializeWith: anInteger! !

!WARouteCountainer class methodsFor: 'instance creation' stamp: 'pmm 9/8/2009 22:10'!
routes: aCollectionOfRoutes
	^ (self pathElementCount: aCollectionOfRoutes anyOne pathElementCount)
		addRoutes: aCollectionOfRoutes;
		yourself! !

!WARouteCountainer methodsFor: 'public' stamp: 'pmm 9/8/2009 17:24'!
addRoute: aRoute
	^ routes add: aRoute! !

!WARouteCountainer methodsFor: 'public' stamp: 'pmm 9/8/2009 22:10'!
addRoutes: aCollectionOfRoutes
	aCollectionOfRoutes do: [ :each |
		self addRoute: each ]! !

!WARouteCountainer methodsFor: 'initialize-release' stamp: 'pmm 9/8/2009 17:24'!
initializeWith: anInteger
	super initialize.
	pathElementCount := anInteger.
	routes := SortedCollection sortBlock: [ :a :b |
		a priority > b priority ]! !

!WARouteCountainer methodsFor: 'public' stamp: 'pmm 9/8/2009 17:26'!
pathElementCount
	^ pathElementCount! !

!WARouteCountainer methodsFor: 'public' stamp: 'pmm 9/9/2009 19:53'!
routeForElements: aCollection parameters: aDictionary ifAbsent: aNiliadicBlock
	^ routes
		detect: [ :each |
			(each matchesPath: aCollection)
				and: [ each matchesParameters: aDictionary ] ]
		ifNone: aNiliadicBlock
	! !

WAObject subclass: #WARouteResult
	instanceVariableNames: 'route elements parameters'
	classVariableNames: ''
	poolDictionaries: ''
	category: 'Seaside-REST-Core-Route'!

!WARouteResult class methodsFor: 'instance creation' stamp: 'pmm 9/8/2009 22:18'!
route: aRoute elements: anArray parameters: aDictionary
	^self basicNew initializeWithRoute: aRoute elements: anArray parameters: aDictionary! !

!WARouteResult methodsFor: 'accessing' stamp: 'pmm 9/2/2009 11:08'!
elements
	^ elements! !

!WARouteResult methodsFor: 'initialize-release' stamp: 'pmm 9/2/2009 11:10'!
initializeWithRoute: aRoute elements: anArray
	super initialize.
	route := aRoute.
	elements := anArray! !

!WARouteResult methodsFor: 'initialize-release' stamp: 'pmm 9/8/2009 22:18'!
initializeWithRoute: aRoute elements: anArray parameters: aDictionary
	super initialize.
	route := aRoute.
	elements := anArray.
	parameters := aDictionary! !

!WARouteResult methodsFor: 'accessing' stamp: 'pmm 9/2/2009 11:08'!
route
	^ route! !

!WARouteResult methodsFor: 'public' stamp: 'pmm 9/9/2009 19:31'!
sendMessageTo: anObject
	| arguments |
	arguments := Array new: route argumentCount.
	route mapArgumentsWithElements: elements parameters: parameters into: arguments.
	^ route
		sendSelectorWith: arguments
		to: anObject! !
